package com.hs.admin.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hs.admin.common.Constants;
import com.hs.admin.common.base.PageRequest;
import com.hs.admin.common.base.PageResult;
import com.hs.admin.common.utils.DateUtils;
import com.hs.admin.common.utils.PageUtil;
import com.hs.admin.mapper.AuditLogMapper;
import com.hs.admin.model.AuditLog;
import com.hs.admin.model.reqmodel.AuditLogReq;
import com.hs.admin.service.AuditLogService;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;

import static java.util.stream.Collectors.groupingBy;

@Service
public class AuditLogServiceImpl implements AuditLogService {

    @Resource
    AuditLogMapper auditLogMapper;

    @Override
    public int add(AuditLog auditLog) {
        int count = auditLogMapper.insert(auditLog);
        return count;
    }

    @Override
    public PageResult<AuditLog> getAllAuditLog(AuditLogReq req) {
        PageRequest page = req.getPage();
        PageHelper.startPage(page.getPageNum(), page.getPageSize());
        String startDate = req.getStartDate() == null?null:(req.getStartDate() + Constants.PREFIX_END_DATE);
        String endDate = req.getEndDate() == null?null:(req.getEndDate() + Constants.SUFFIX_END_DATE);
        List<AuditLog> allAuditLogs = auditLogMapper.getAllAuditLogs(startDate, endDate);
        PageResult pageResult = PageUtil.getPageResult(page, new PageInfo<AuditLog>(allAuditLogs));
        getOnlineTime(pageResult.getContent());
        return pageResult;
    }

    private void getOnlineTime(List<AuditLog> auditLogs) {
        for(AuditLog auditLog: auditLogs) {
            List<AuditLog> tokenList = auditLogMapper.getAllTokens(auditLog.getToken());
            Date loginTime = null;
            Date logoutTime = null;
            Date lastRefreshTIme = null;

            for(AuditLog tokenAuditLog: tokenList) {
                Date createDate = tokenAuditLog.getCreateDate();
                if(tokenAuditLog.getOperateType().equals("login")) {
                    loginTime = createDate;
                }else if(tokenAuditLog.getOperateType().equals("logout")) {
                    logoutTime = createDate;
                }else if(tokenAuditLog.getOperateType().equals("refresh")) {
                    if(lastRefreshTIme==null || createDate.getTime()>lastRefreshTIme.getTime()) {
                        lastRefreshTIme = createDate;
                    }
                }
            }
            //有登录、退出时间的
            if(loginTime!=null && logoutTime!=null) {
                String dateMinus = DateUtils.dateDiff(loginTime, logoutTime);
                auditLog.setOnlineTime(dateMinus);
            }else if(loginTime!=null && logoutTime==null && lastRefreshTIme!=null) {
                String dateMinus = DateUtils.dateDiff(loginTime, lastRefreshTIme);
                auditLog.setOnlineTime(dateMinus);
            }else{
                auditLog.setOnlineTime(Constants.DEFAULT_ONLINE_TIME);
            }
        }

    }


}
