package com.hs.admin.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hs.admin.common.base.PageRequest;
import com.hs.admin.common.base.PageResult;
import com.hs.admin.common.enums.StateType;
import com.hs.admin.common.utils.AESUtil;
import com.hs.admin.common.utils.PageUtil;
import com.hs.admin.mapper.SysUserMapper;
import com.hs.admin.model.SysUser;
import com.hs.admin.model.reqmodel.AddUserReq;
import com.hs.admin.model.reqmodel.UpdateUserReq;
import com.hs.admin.model.reqmodel.UserReq;
import com.hs.admin.model.respmodel.SysUserList;
import com.hs.admin.service.DicOrgService;
import com.hs.admin.service.SysRoleService;
import com.hs.admin.service.UserService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class UserServiceImpl implements UserService {

    @Resource
    private SysUserMapper sysUserMapper;
    @Resource
    private SysRoleService sysRoleService;
    @Resource
    private DicOrgService dicOrgService;

    @Override
    public SysUser findByLoginName(String loginName) {
        return sysUserMapper.selectByCode(loginName);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean addUser(Long userId, AddUserReq user) {
        if (user != null) {
            //添加用户信息
            SysUser sysUser = new SysUser();
            sysUser.setUserCode(user.getUserCode());
            sysUser.setUserName(user.getUserName());
            sysUser.setPassword(AESUtil.encrypt(user.getPassword()));
            sysUser.setIdcard(user.getIdcard());
            sysUser.setMobile(user.getMobile());
            sysUser.setSex(user.getSex());
            sysUser.setCreateDate(new Date());
            sysUser.setLastUpdatePwdTime(new Date());
            sysUser.setCreateUserid(userId);
            sysUser.setState((short) StateType.ON.getValue());
            sysUser.setIsAdmin(user.getIsAdmin());
            sysUser.setIsRoleylz(user.getIsRoleylz());  //add by mengchao 20210521
            sysUser.setIsLeader(user.getIsLeader());  //add by mengchao 20210521
            sysUserMapper.insert(sysUser);

            //添加角色信息
            sysRoleService.saveUserRole(userId, (long) sysUser.getId(), user.getRoleId());

            //添加科室信息
            dicOrgService.saveUserOrg(userId, (long) sysUser.getId(), user.getOrg());

            return true;
        }
        return false;
    }

    @Override
    public boolean deleteUser(UserReq.DeleteUserReq user) {
        if (user != null) {
            SysUser sysUser = sysUserMapper.selectByPrimaryKey(user.getUserId());
            if (sysUser != null) {
                sysUser.setState(sysUser.getState().equals((short) StateType.OFF.getValue()) ? (short) StateType.ON.getValue() : (short) StateType.OFF.getValue());
                sysUserMapper.updateByPrimaryKey(sysUser);
                return true;
            }
        }
        return false;
    }

    @Override
    public boolean unlockUser(UserReq.UnlockUserReq user) {
        if (user != null) {
            SysUser sysUser = sysUserMapper.selectByPrimaryKey(user.getUserId());
            if (sysUser != null) {
                sysUser.setLockFlag(false);
                sysUserMapper.updateByPrimaryKey(sysUser);
                return true;
            }
        }
        return false;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateUser(UpdateUserReq user) {
        if (user != null) {
            SysUser sysUser = sysUserMapper.selectByPrimaryKey(user.getUserId());
            if (sysUser != null) {
                //修改用户
                sysUser.setUserCode(user.getUserCode());
                sysUser.setUserName(user.getUserName());
                sysUser.setPassword(AESUtil.encrypt(user.getPassword()));
                sysUser.setLastUpdatePwdTime(new Date());
                sysUser.setIdcard(user.getIdcard());
                sysUser.setMobile(user.getMobile());
                sysUser.setSex(user.getSex());
                sysUser.setIsAdmin(user.getIsAdmin());
                sysUser.setIsRoleylz(user.getIsRoleylz());  //add by mengchao 20210521
                sysUser.setIsLeader(user.getIsLeader());  //add by mengchao 20210521
                sysUserMapper.updateByPrimaryKey(sysUser);

                //修改角色信息
                sysRoleService.saveUserRole(1l, (long) sysUser.getId(), user.getRoleId());

                //修改科室信息
                dicOrgService.saveUserOrg(1l, (long) sysUser.getId(), user.getOrg());

                return true;
            }
        }
        return false;
    }

    @Override
    public List<SysUserList> getAll(UserReq.GetUserReq user) {
        List<SysUserList> list = sysUserMapper.selectUserList();
        if (user.getUserName() != null) {
            list = list.stream().filter(
                    o -> (o.getUserName() != null && o.getUserName().toLowerCase().contains(user.getUserName().toLowerCase())))
                    .collect(Collectors.toList());
        }
        return list;
    }

    @Override
    public PageResult getAllByPage(UserReq.GetUserReq user) {
        PageRequest page = user.getPage();
        PageHelper.startPage(page.getPageNum(), page.getPageSize());
        List<SysUserList> sysMenus = sysUserMapper.selectUserList();
        /*List<SysUserList> sysUserLists = new ArrayList<>();
        sysMenus.forEach(sysUserList -> {
            String password = sysUserList.getPassword();
            String encrypt = AESUtil.encrypt(password);
            sysUserList.setPassword(encrypt);
            sysUserLists.add(sysUserList);
        });*/
        PageResult pageResult = PageUtil.getPageResult(page, new PageInfo<SysUserList>(sysMenus));
        return pageResult;
    }

    @Override
    public SysUserList info(Integer userId) {
        SysUserList sysUser = sysUserMapper.selectUserById(userId);
        /*sysUser.setPassword(AESUtil.encrypt(sysUser.getPassword()));*/
        return sysUser;
    }
    @Override
    public boolean updateSysconfig(String state) {
        if (sysUserMapper.updateconfig(state)>0 ) {
                          return true;
            }

        return false;
    }

    @Override
    public String selectpwdcontol() {
        return sysUserMapper.selectpwdcontol();
    }

}
