package com.hs.api.common.aspect;

import com.hs.api.common.Result;
import com.hs.api.common.annotations.Audit;
import com.hs.api.common.enums.AuditLogType;
import com.hs.api.model.AuditLog;
import com.hs.api.model.SysUser;
import com.hs.api.model.respmodel.LoginInfo;
import com.hs.api.service.AuditLogService;
import com.hs.api.service.UserService;
import com.hs.common.utils.HttpUtil;
import com.hs.common.utils.RedisUtil;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Aspect
@Component
public class AuditAspect {

    @Resource
    AuditLogService auditLogService;

    @Resource
    private UserService userServiceImpl;

    @Resource
    AuditLog auditLog;

    @Pointcut("@annotation(com.hs.api.common.annotations.Audit)")
    public void audit() {

    }

    @Before("audit()")
    public void beforeExec(JoinPoint joinPoint) {
    }

    @After("audit()")
    public void afterExec(JoinPoint joinPoint) {
    }

    @Around("audit()")
    public Object aroundExec(ProceedingJoinPoint pjp) throws Throwable {
        Object result = pjp.proceed();
        return result;
    }

    /**
     * 带参返回
     */
    @AfterReturning(pointcut = "audit()", returning = "rc")
    public void afterReturning(JoinPoint joinPoint, Result rc) {
        if(rc.getCode() == Result.ErrorCode.SUCCESS.getCode()) {
            MethodSignature ms = (MethodSignature) joinPoint.getSignature();
            Method method = ms.getMethod();
            Audit audit = method.getAnnotation(Audit.class);
            AuditLogType auditType = audit.type();
            int moduleCode = auditType.getModuleCode();
            String operateType = auditType.getOperateType();
            String describe = auditType.getDescribe();
            SysUser user = new SysUser();
            String token = new String();

            HttpServletRequest request = ((ServletRequestAttributes)RequestContextHolder.getRequestAttributes()).getRequest();
            String ip = HttpUtil.getIP(request);
            String uri = request.getRequestURI();
            Object[] params = joinPoint.getArgs();
            List<Object> paramsList = Arrays.asList(params);

            if(operateType.equals(AuditLogType.REFRESH.getOperateType())) {
                token = request.getHeader("Authorization");
                user = (SysUser) RedisUtil.get(token);
            }else {
                LoginInfo loginInfo = (LoginInfo) rc.getData();
                user = loginInfo.getUser();
                token = loginInfo.getToken();
                if(user == null) {
                    return;
                }
            }

            auditLog.setCreateDate(new Date());
            auditLog.setState(true);
            auditLog.setUserId(user.getId());
            auditLog.setIp(ip);
            auditLog.setModuleCode(moduleCode);
            auditLog.setOperateType(operateType);
            auditLog.setDescribe(describe);
            auditLog.setToken(token);
            auditLog.setUrl(uri);
            auditLog.setParams(paramsList.toString());
            auditLogService.add(auditLog);
        }
    }

}
