package com.hs.api.common.intercepters;

import com.hs.api.common.Constants;
import com.hs.api.common.Result;
import com.hs.api.common.enums.SysConfigKeyType;
import com.hs.api.common.utils.TokenUtil;
import com.hs.api.model.SysUser;
import com.hs.api.service.SysConfigService;
import com.hs.common.utils.RedisUtil;
import com.hs.common.utils.StringUtil;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.List;

public class LoginInterceptor implements HandlerInterceptor {
    Logger logger = org.slf4j.LoggerFactory.getLogger(LoginInterceptor.class);

    @Autowired
    private SysConfigService sysConfigService;

    @Override
    public boolean preHandle(HttpServletRequest request,HttpServletResponse response,Object handler) throws Exception {
        System.out.println("LoginInterceptor----------->preHandle");

        String token = request.getHeader(Constants.TOKEN_KEY);
        if(!TokenUtil.checkToken(token)){
            return doResponse(request, response, token, Result.ErrorCode.INVALID_TOKEN);
        }else if(sysConfigService.getStateByKey(SysConfigKeyType.SIMULTAN_ONLINE_NUM.getCode())){
            //如果已登录的tokenList不包含该token 则报错
            String path = request.getServletContext().getContextPath();
            SysUser user = (SysUser) RedisUtil.get(token);
            List<String> tokenList = (List<String>)RedisUtil.get(user.getId() + path);
            if(tokenList!=null && !tokenList.contains(token)){
                return doResponse(request, response, token, Result.ErrorCode.KICK_OUT_TOKEN);
            }
        }

        return HandlerInterceptor.super.preHandle(request,response,handler);
    }

    private boolean doResponse(HttpServletRequest request, HttpServletResponse response, String token, Result.ErrorCode errorCode) {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        PrintWriter writer = null;
        try{
            String remoteHost = request.getRemoteHost();
            String uri = request.getRequestURI();
            logger.info(remoteHost + " 访问 " + uri + ", token无效, token：[" + token + "]");

            writer = response.getWriter();
            writer.print(Result.error(errorCode));
        }catch(IOException e){
            logger.error(e.getMessage());
        }finally{
            if(writer != null){
                writer.close();
            }
        }
        return false;
    }

    @Override
    public void postHandle(HttpServletRequest request,HttpServletResponse response,Object handler,ModelAndView modelAndView) throws Exception {
        System.out.println("LoginInterceptor----------->postHandle");


        HandlerInterceptor.super.postHandle(request,response,handler,modelAndView);
    }

    @Override
    public void afterCompletion(HttpServletRequest request,
                                HttpServletResponse response,Object handler,Exception ex) throws Exception {
        System.out.println("LoginInterceptor------->afterCompletion");


        HandlerInterceptor.super.afterCompletion(request,response,handler,ex);
    }
}
