package com.hs.api.controller;

import com.hs.api.common.Constants;
import com.hs.api.common.Result;
import com.hs.api.common.annotations.Audit;
import com.hs.api.common.annotations.CurrentUser;
import com.hs.api.common.annotations.Token;
import com.hs.api.common.enums.AuditLogType;
import com.hs.api.model.SerDepartment;
import com.hs.api.model.SysRole;
import com.hs.api.model.SysUser;
import com.hs.api.model.reqmodel.Disease;
import com.hs.api.model.reqmodel.MenuReq;
import com.hs.api.model.reqmodel.ModuleReq;
import com.hs.api.model.respmodel.LoginInfo;
import com.hs.api.model.respmodel.SysMenuList;
import com.hs.api.service.*;
import com.hs.common.utils.RedisUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@Api(tags = "用户API", produces = "produces", consumes = "consumes", protocols = "protocols")
@RequestMapping("/user")
@RestController
public class UserController {
    @Autowired
    private SysUserRoleRsService sysUserRoleRsService;
    @Autowired
    private SysMenuService sysMenuService;
    @Autowired
    private SysUserOrgRsService sysUserOrgRsService;
    @Autowired
    private LoginService loginServiceImpl;

    @Autowired
    private UserService userService;

    /**
     * 根据token获取用户信息
     * @return
     */
    @GetMapping("/getByToken")
    @ApiIgnore
    public Object getByToken(HttpServletRequest request) {
        String token = request.getHeader(Constants.TOKEN_KEY);
        SysUser user = (SysUser) RedisUtil.get(token);
//        return Result.success(user);

        LoginInfo loginInfo = loginServiceImpl.login(
                user.getUserCode(), user.getPassword(), user.getMacAddress(), user.getLastIp());
        if (loginInfo == null) {
            return Result.error(Result.ErrorCode.USERERROR);
        }
        return Result.success(loginInfo);
    }

    @GetMapping()
    @ApiIgnore
    public Object user(@CurrentUser @ApiIgnore SysUser user) {
        return Result.success(user);
    }


    /**
     * 根据用户id查询用户角色
     *
     * @param userId 用户id
     * @return
     */
    @PostMapping("roles")
    @Token
    @Audit(type = AuditLogType.REFRESH)
    @ApiOperation("查询用户角色")
    public Object getRoleListByUser(@ApiIgnore @CurrentUser Long userId) throws Exception {
        List<SysRole> sysRoleList = sysUserRoleRsService.getRoleListByUser(userId);
        return Result.success(sysRoleList);
    }

    /**
     * 根据用户id查询用户菜单
     *
     * @param userId 用户id
     * @return
     */
    @PostMapping("menus")
    @Token
    @Audit(type = AuditLogType.REFRESH)
    @ApiOperation("查询用户菜单")
    public Object getMenuByUser(@ApiIgnore @CurrentUser Long userId) throws Exception {
        List<SysMenuList> sysMenuList = sysMenuService.getMenu(userId);
        return Result.success(sysMenuList);
    }

    @PostMapping("menu/list")
    @Token
    @Audit(type = AuditLogType.REFRESH)
    @ApiOperation("查询综合菜单")
    public Object getMenuByType(HttpServletRequest request,@RequestBody MenuReq req) throws Exception {
        String token = request.getHeader(Constants.TOKEN_KEY);
        SysUser user = (SysUser) RedisUtil.get(token);
        List<SysMenuList> sysMenuList = sysMenuService.getMenuByType(user.getId(), req.getMenuType());
        return Result.success(sysMenuList);
    }

    @PostMapping("module/state")
    @Token
    @Audit(type = AuditLogType.REFRESH)
    @ApiOperation("查询module状态")
    public Object getModuleStateByCode(HttpServletRequest request,@RequestBody ModuleReq req) throws Exception {
        String token = request.getHeader(Constants.TOKEN_KEY);
        SysUser user = (SysUser) RedisUtil.get(token);
        boolean menuState = sysMenuService.getModuleStateByCode(user.getId(), req.getModuleCode());
        return Result.success(menuState);
    }

    /**
     * 根据用户id查询用户机构
     *
     * @param userId 用户id
     * @return
     */
    @PostMapping("dept")
    @Token
    @Audit(type = AuditLogType.REFRESH)
    @ApiOperation("查询用户科室")
    public Object getOrgByUser(@ApiIgnore @CurrentUser Long userId, @RequestBody Disease.DiseaseIDorLevel disease) throws Exception {
        List<SerDepartment> sysMenuList = sysUserOrgRsService.getUserOrg(userId, disease.getDisease(), disease.getDate());
        return Result.success(sysMenuList);
    }

    /**
     * 初始化用户密码
     *
     * @return
     */
    @GetMapping("init")
    @ApiOperation("批量初始化密码")
    public Object init() throws Exception {
        int count = userService.init();
        return Result.success(count);
    }
}
